@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013-2014 Jolla Ltd.
+** Copyright (C) 2013-2015 Jolla Ltd.
 ** Contact: lorn.potter@jollamobile.com
 **
 ** GNU Lesser General Public License Usage
@@ -28,8 +28,10 @@ class QOFONOSHARED_EXPORT QOfonoConnectionContext : public QOfonoObject
 {
     Q_OBJECT
     Q_PROPERTY(bool active READ active WRITE setActive NOTIFY activeChanged)
+    Q_PROPERTY(bool provisioning READ provisioning NOTIFY provisioningChanged)
     Q_PROPERTY(QString accessPointName READ accessPointName WRITE setAccessPointName NOTIFY accessPointNameChanged)
     Q_PROPERTY(QString type READ type WRITE setType NOTIFY typeChanged)
+    Q_PROPERTY(QString authMethod READ authMethod WRITE setAuthMethod NOTIFY authMethodChanged)
     Q_PROPERTY(QString username READ username WRITE setUsername NOTIFY usernameChanged)
     Q_PROPERTY(QString password READ password WRITE setPassword NOTIFY passwordChanged)
     Q_PROPERTY(QString protocol READ protocol WRITE setProtocol NOTIFY protocolChanged)
@@ -52,12 +54,17 @@ public:
     bool active() const;
     void setActive(bool);
 
+    bool provisioning() const;
+
     QString accessPointName() const;
     void setAccessPointName(const QString &);
 
     QString type() const;
     void setType(const QString &);
 
+    QString authMethod() const;
+    void setAuthMethod(const QString &);
+
     QString username() const;
     void setUsername(const QString &);
 
@@ -82,22 +89,26 @@ public:
     bool isValid() const;
     QString modemPath() const;
 
-    Q_INVOKABLE bool validateProvisioning(); //check provision against mbpi
-    Q_INVOKABLE bool validateProvisioning(const QString &provider, const QString &mcc, const QString &mnc); //check provision against mbpi
+    bool validateProvisioning(); //check provision against mbpi
+    bool validateProvisioning(const QString &provider, const QString &mcc, const QString &mnc); //check provision against mbpi
     #if QT_VERSION < 0x050000
-    Q_INVOKABLE void provision(const QString &provider, const QString &mcc, const QString &mnc, const QString &type="internet");
+    void provision(const QString &provider, const QString &mcc, const QString &mnc, const QString &type="internet");
     #else
-    Q_INVOKABLE void provision(const QString &provider, const QString &mcc, const QString &mnc, const QString &type=QStringLiteral("internet")); // provision context against mbpi
+    void provision(const QString &provider, const QString &mcc, const QString &mnc, const QString &type=QStringLiteral("internet")); // provision context against mbpi
     #endif
-    Q_INVOKABLE void provisionForCurrentNetwork(const QString &type);
+    void provisionForCurrentNetwork(const QString &type);
+
     Q_INVOKABLE void disconnect();
+    Q_INVOKABLE bool provision();
 
 Q_SIGNALS:
     void disconnectRequested();
     void activeChanged(bool active);
+    void provisioningChanged(bool provisioning);
     void accessPointNameChanged(const QString &apn);
     void nameChanged(const QString &name);
     void typeChanged(const QString &type);
+    void authMethodChanged(const QString &method);
     void usernameChanged(const QString &uname);
     void passwordChanged(const QString &pw);
     void protocolChanged(const QString &proto);
@@ -106,14 +117,22 @@ Q_SIGNALS:
     void settingsChanged(const QVariantMap &settingsMap);
     void IPv6SettingsChanged(const QVariantMap &ipv6SettingsMap);
     void contextPathChanged(const QString &contextPath);
-    void provisioningFinished();
+    void provisioningFinished(const QString &error);
     void modemPathChanged(const QString &path);
 
+private Q_SLOTS:
+    void onProvisionContextFinished(QDBusPendingCallWatcher *watch);
+    void onManagerValidChanged(bool valid);
+
 protected:
     QDBusAbstractInterface *createDbusInterface(const QString &path);
     QVariant convertProperty(const QString &key, const QVariant &value);
     void propertyChanged(const QString &key, const QVariant &value);
     void objectPathChanged(const QString &path, const QVariantMap *properties);
+
+private:
+    class Private;
+    Private *privateData() const;
 };
 
 #endif // QOFONOCONNECTIONCONTEXT_H
