@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013-2014 Jolla Ltd.
+** Copyright (C) 2013-2015 Jolla Ltd.
 ** Contact: lorn.potter@jollamobile.com
 **
 ** GNU Lesser General Public License Usage
@@ -14,7 +14,7 @@
 ****************************************************************************/
 
 #include "qofonomanager.h"
-#include "dbus/ofonomanager.h"
+#include "ofono_manager_interface.h"
 
 class QOfonoManager::Private
 {
@@ -22,9 +22,8 @@ public:
     OfonoManager *ofonoManager;
     QStringList modems;
     bool available;
-    QDBusServiceWatcher *ofonoWatcher;
 
-    Private() : ofonoManager(NULL), available(false), ofonoWatcher(NULL) {}
+    Private() : ofonoManager(NULL), available(false) {}
 };
 
 QOfonoManager::QOfonoManager(QObject *parent) :
@@ -33,17 +32,16 @@ QOfonoManager::QOfonoManager(QObject *parent) :
 {
     QOfonoDbusTypes::registerObjectPathProperties();
     QDBusConnection systemBus(QDBusConnection::systemBus());
-    d_ptr->ofonoWatcher = new QDBusServiceWatcher("org.ofono", systemBus,
+    QDBusServiceWatcher *ofonoWatcher = new QDBusServiceWatcher("org.ofono", systemBus,
             QDBusServiceWatcher::WatchForRegistration |
             QDBusServiceWatcher::WatchForUnregistration, this);
 
-    connect(d_ptr->ofonoWatcher, SIGNAL(serviceRegistered(QString)),
+    connect(ofonoWatcher, SIGNAL(serviceRegistered(QString)),
             this, SLOT(connectToOfono(QString)));
-    connect(d_ptr->ofonoWatcher, SIGNAL(serviceUnregistered(QString)),
+    connect(ofonoWatcher, SIGNAL(serviceUnregistered(QString)),
             this, SLOT(ofonoUnregistered(QString)));
 
-    d_ptr->available = systemBus.interface()->isServiceRegistered("org.ofono");
-    if (d_ptr->available) {
+    if (systemBus.interface()->isServiceRegistered("org.ofono")) {
         connectToOfono(QString());
     }
 }
@@ -70,7 +68,9 @@ bool QOfonoManager::available() const
 
 bool QOfonoManager::isValid() const
 {
-    return d_ptr->ofonoManager && d_ptr->ofonoManager->isValid();
+    // isValid() is essentially the same as available(), keeping it around for
+    // backward compatibility
+    return d_ptr->available;
 }
 
 void QOfonoManager::onModemAdded(const QDBusObjectPath& path, const QVariantMap&)
@@ -79,6 +79,7 @@ void QOfonoManager::onModemAdded(const QDBusObjectPath& path, const QVariantMap&
     if (!d_ptr->modems.contains(pathStr)) {
         QString prevDefault = defaultModem();
         d_ptr->modems.append(pathStr);
+        qSort(d_ptr->modems);
         Q_EMIT modemAdded(pathStr);
         Q_EMIT modemsChanged(d_ptr->modems);
         QString newDefault = defaultModem();
@@ -106,36 +107,28 @@ void QOfonoManager::onGetModemsFinished(QDBusPendingCallWatcher* watcher)
 {
     QDBusPendingReply<ObjectPathPropertiesList> reply(*watcher);
     if (reply.isValid() && !reply.isError()) {
-        // fugly I know... but we need sorted modems
-        // with hardware listed first
         QString prevDefault = defaultModem();
-        d_ptr->modems.clear();
-        foreach (ObjectPathProperties modem, reply.value()) {
-            QString modemPath = modem.path.path();
-            QString modemType = modem.properties["Type"].value<QString>();
-            if (modemType == "hardware" && !modemPath.contains("phonesim")) {
-                // running phonesim from desktop presents phonesim as hardware
-                d_ptr->modems.prepend(modemPath);
-            } else {
-                d_ptr->modems.append(modemPath);
-            }
-            Q_EMIT modemAdded(modemPath);
+        QStringList newModems;
+        Q_FOREACH(ObjectPathProperties modem, reply.value()) {
+            newModems.append(modem.path.path());
+        }
+        qSort(newModems);
+        if (d_ptr->modems != newModems) {
+            d_ptr->modems = newModems;
+            Q_EMIT modemsChanged(d_ptr->modems);
         }
-        Q_EMIT modemsChanged(d_ptr->modems);
         QString newDefault = defaultModem();
         if (newDefault != prevDefault) {
             Q_EMIT defaultModemChanged(newDefault);
         }
+        d_ptr->available = true;
+        Q_EMIT availableChanged(true);
     }
     watcher->deleteLater();
 }
 
 void QOfonoManager::connectToOfono(const QString &)
 {
-    if (!d_ptr->available) {
-        d_ptr->available = true;
-        Q_EMIT availableChanged(true);
-    }
     if (!d_ptr->ofonoManager) {
         OfonoManager* mgr = new OfonoManager("org.ofono", "/", QDBusConnection::systemBus(), this);
         if (mgr->isValid()) {
